
CatDPSTest = true
if(Tests == nil) then
	SkipTests=true
	require('test')
	SkipTests=false
end

require('CatDPS')


local Player = CatPoints.Modifiers['CatDPS-2.3']:new()


local function dummy(value)
	return function() return value end 
end

function Tests.ZeroHaste()
	local actual = 1-Player:new({Haste=0}).AttackSpeed
	local expected = 1

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end


Tests.OneHaste = function()
	local actual = Player:new({Haste=15.76}).AttackSpeed
	local expected = .01

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Expertise = function()
	local actual = Player:new({Expertise = 4*3.92}).DodgeReduction
	local expected = .01

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Melee = function()
		local base = Player:new({AttackPower=0,WeaponDamage=0})
		local plus14AP = Player:new({AttackPower=14, WeaponDamage=0})
		local plus1Weapon = Player:new({WeaponDamage=1, AttackPower=0})

		return TEST(plus14AP.Melee==plus1Weapon.Melee and plus14AP.Melee==(base.Melee+1), 
			format("Expected melee values to match: %.2f, %.2f, %.2f", base.Melee, plus14AP.Melee, plus1Weapon.Melee))
	end

Tests.HitRating= function()
	local actual = Player:new({Hit = 15.77}).MissReduction
	local expected = .01

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.SavageFury = function()
	local expected = Player:new({SavageFury=0, WeaponDamage=0, AttackPower=0, Idol='None'}).MangleDamage*1.2
	local actual = Player:new({SavageFury=2, WeaponDamage=0, AttackPower=0, Idol='None'}).MangleDamage

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

Tests.Mangle = function()
	local expected = 2*1.03*1.1*(1.6+264+24)/40 
	local player = Player:new({Diamond=true,Melee=1, Idol='IdoloftheWild', Ferocity=5, PredatoryInstincts=5,})
	
	local actual = player.MangleCritDamage/player.MangleCost

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end
	
Tests.SavageFury = function()
	local expected = Player:new({            Idol='None', WeaponDamage=0, AttackPower=0}).MangleDamage*1.2
	local actual = Player:new({SavageFury=2, Idol='None', WeaponDamage=0, AttackPower=0}).MangleDamage

	local error = PercentError(actual, expected)
	return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
end

	Tests.Shred = function()
		local expected = 2*(1.1)*1.03*(1.3*(2.25+405)+88+75)/42
		local player = Player:new({Diamond=true, Melee=1, Nordrassil=4, Idol='EverbloomIdol', ShreddingAttacks=2, PredatoryInstincts=5})
		local actual = player.ShredCritDamage/player.ShredCost

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
	end
	
	Tests.Rip = function()
		local expected = 1.15*1.3*(1553 + .24 + 120)
		local player = Player:new({AttackPower=1,Idol='IdolofFeralShadows', Thunderheart=4})
		local actual = player.RipDamage(5)

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
	end
	
	Tests.HeartOfTheWild = function()
		local expected = 11
		local player = Player:new({HeartoftheWild=5})
		player.derivedStats = {
			AttackPower=dummy(0),
			Crit = dummy(0),
			Hit = dummy(0),
			Haste = dummy(0),
			WeaponDamage=dummy(0),
			Expertise=dummy(0),
		}
		local after=player:applyBonus({ATTACKPOWERFERAL=5, ATTACKPOWER=5})
		local actual = after.AttackPower - player.AttackPower

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
	end
	
	Tests.Strength = function()
		local expected = 5
		local player = Player:new({HeartoftheWild=5, SurvivaloftheFittest=3})
		
		player.derivedStats = {
			AttackPower=dummy(0),
			Crit = dummy(0),
			Hit = dummy(0),
			Haste = dummy(0),
			WeaponDamage=dummy(0),
			Expertise=dummy(0),
		}
		local after=player:applyBonus({STR=expected})

		--* sotf * ap per str * HOTW
		expected = expected * 1.03 * 2 *1.1
		local actual = after.AttackPower 

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
	end

	Tests.AgiAP = function()
		local expected = 5
		local player = Player:new({HeartoftheWild=5, SurvivaloftheFittest=3})
		
		player.derivedStats = {
			AttackPower=dummy(0),
			Crit = dummy(0),
			Hit = dummy(0),
			Haste = dummy(0),
			WeaponDamage=dummy(0),
			Expertise=dummy(0),
		}
		local after=player:applyBonus({AGI=expected})

		--* sotf * ap per str * HOTW
		expected = expected * 1.03 * 1 *1.1
		local actual = after.AttackPower 

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.2f and got %.2f (%.3f%% difference)", expected, actual,error))
	end
	
	
	
	Tests.Crit = function()
		local expected = 25
		local player = Player:new({SurvivaloftheFittest=3, AttackPower=0})
		
		player.derivedStats = {
			AttackPower=dummy(0),
			Crit = dummy(0),
			Hit = dummy(0),
			Haste = dummy(0),
			WeaponDamage=dummy(0),
			Expertise=dummy(0),
		}
		
		after=player:applyBonus({AGI=expected, CR_CRIT=22.1, AttackPower=0})

		--* sotf * ap per agi * HOTW
		expected = (expected*1.03)/2500 + .01
		local actual = after.Crit - player.Crit

		local error = PercentError(actual, expected)
		return TEST(error < 1, format("Expected %.5f and got %.5f (%.3f%% difference)", expected, actual,error))
	end	
	
if(not SkipTests) then
	RunTests(Tests, false)
end