


local function print(x)
	if(type(x)=="table") then
		for k,v in pairs(x) do
			ChatFrame1:AddMessage(k.." = "..tostring(v))
		end
	else
		ChatFrame1:AddMessage(x)
	end
end


local Options = {
	Enemy = {
		Level=73,
		Miss=.05,
	},
	BaseStat='STA',

	derivedStats = {
		Idol = function()
			return CatPoints:GetIdol()
		end,
		Health = function()
			return UnitHealthMax('player')
		end,
		Armor = function()
			return UnitArmor('player')
		end,
		HpPerStamina = function()
			local _,race = UnitRace('player')
			if(race == 'Tauren') then
				return 10.5
			else
				return 10
			end
		end,
		ArmorMultiplier = function(player)
			return 5.0*(1 + 0.1*player.ThickHide/3)
		end,
		Defense = function()
			local baseDefense, armorDefense = UnitDefense('player')
			return (baseDefense+armorDefense)
		end,
		Resilience = function()
			return GetCombatRating(CR_CRIT_TAKEN_MELEE)
		end,
		Dodge = function()
			return GetDodgeChance()/100
		end,
		Level = function()
			return UnitLevel('player')
		end,
	},
}

function Options:display()
	return GetShapeshiftForm(true)==1
end

function Options:applyBonus(bonuses, considerSet)
	local relevant = nil
	local s = self:new({}, self.tableName..'.applyBonus')
	if(#bonuses == 0) then
		relevant = {}
	end

	local sotf = (1+s.SurvivaloftheFittest*.01)

	setmetatable(bonuses, {__index = function(_, name) if(relevant) then relevant[name]=true end return 0 end})

	s.Health = s.Health +bonuses.HEALTH+bonuses.STA*sotf*1.25*(1 + .2*s.HeartoftheWild/5)*s.HpPerStamina

	s.Defense = s.Defense + bonuses.CR_DEFENSE/2.4
	s.Resilience = s.Resilience +bonuses.CR_RESILIENCE
	s.Dodge = s.Dodge + bonuses.CR_DODGE/1892 + bonuses.AGI/1471

	s.Armor = s.Armor + 2*bonuses.AGI*sotf+bonuses.BASE_ARMOR*s.ArmorMultiplier+bonuses.ARMOR_BONUS

	if(bonuses.Idol and bonuses.Idol ~= 0) then
		s.Idol = bonuses.Idol
	end
	return s,relevant
end

function Options:display()
	return GetShapeshiftForm(true)==1
end

function Options:calculate()

	local defenseEffect = (self.Defense- self.Level *5)*.0004
	

	local resilenceEffect = self.Resilience/3940
	
	local attackTable = {
		miss = self.Enemy.Miss+defenseEffect,
		dodge = self.Dodge,
		--no parry
		--no glancing blow
		--no block
		crit = .05 + (self.Enemy.Level - self.Level)*.002 - self.SurvivaloftheFittest*.01 - resilenceEffect - defenseEffect,
		crushing = (self.Enemy.Level-self.Level) * .05,
	}
	
	
	if(attackTable.crit < 0 ) then
		attackTable.crit = 0
	end
	

	local mobDamage = 1 - attackTable.miss - attackTable.dodge + attackTable.crit*(1-2*resilenceEffect)+.5*attackTable.crushing

	return (self.Health/(mobDamage*(1-CatPoints:MitigationFromArmor(self.Armor))))/100
end


local Player = CatPoints:CreateTable("Tank", Options)